"""
Term level filtering and ``post_filter`` backends.
"""

from .common import FilteringFilterBackend
from .geo_spatial import GeoSpatialFilteringFilterBackend
from .ids import IdsFilterBackend
from .nested import NestedFilteringFilterBackend
from .post_filter import PostFilterFilteringFilterBackend

__title__ = "django_elasticsearch_dsl_drf_alt.filter_backends.filtering"
__author__ = "Artur Barseghyan <artur.barseghyan@gmail.com>"
__copyright__ = "2017-2020 Artur Barseghyan"
__license__ = "GPL 2.0/LGPL 2.1"
__all__ = (
    "FilteringFilterBackend",
    "GeoSpatialFilteringFilterBackend",
    "IdsFilterBackend",
    "NestedFilteringFilterBackend",
    "PostFilterFilteringFilterBackend",
)
